using System.Threading;
using Microsoft.AspNetCore.SignalR;
using Microsoft.Extensions.Logging;
using Todo.Api.Hubs;
using Todo.Core.Application.Services.UserConnectionStore;
using Todo.Core.Interfaces.Publisher;
using Todo.Core.Interfaces.User;

namespace Todo.Api.Publishers;

public class TodoPublisher : ITodoPublisher
{
    private readonly ICurrentUserService _currentUserService;
    private readonly IHubContext<TodoHub> _hubContext;
    private readonly ILogger<TodoPublisher> _logger;
    private readonly IUserConnectionStore _userConnectionStore;

    public TodoPublisher(
        IHubContext<TodoHub> hubContext,
        ICurrentUserService currentUserService,
        IUserConnectionStore userConnectionStore,
        ILogger<TodoPublisher> logger)
    {
        _hubContext = hubContext;
        _currentUserService = currentUserService;
        _userConnectionStore = userConnectionStore;
        _logger = logger;
    }

    public async Task Publish<T>(
        string eventType,
        T message,
        CancellationToken cancellationToken)
    {
        var userId = _currentUserService.GetUserId() ??
                     throw new InvalidOperationException("Cannot proceed without user");
        var connections =
            await _userConnectionStore.GetConnectionsAsync(userId);

        switch (eventType)
        {
            case "todoCreated":
                await _hubContext
                    .Clients
                    .Clients(connections)
                    .SendAsync(
                        "todoCreated",
                        message,
                        cancellationToken);
                break;

            case "todoUpdated":
                await _hubContext
                    .Clients
                    .Clients(connections)
                    .SendAsync(
                        "todoUpdated",
                        message,
                        cancellationToken);
                break;
        }


        _logger.LogInformation(
            "todo event: {EventType} with message:  {Message}",
            eventType,
            message);
    }
}